#!/usr/bin/python3
import datetime
import os
import sys
from PyQt5 import QtGui, QtWidgets, QtCore, QtMultimedia

try:
    from obsuite.backend.common import Slave, trayEventParser
except ImportError:
    from backend.common import Slave, trayEventParser

LOCAL_DIR = os.path.dirname(os.path.realpath(__file__)) + '/'


class Status(QtCore.QObject):
    popup = QtCore.pyqtSignal(str)
    fetch = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.ready = False
        self.slave = parent.slave
        self.preferences = parent.preferences
        self.icon = "default"
        self.lastBrightness = -1
        self.lastLevel = 0
        self.level = 0
        self.dim = False

    def _dim(self):
        start = self.preferences.get("parameters", "brightness", "dim start")
        stop = self.preferences.get("parameters", "brightness", "dim end")
        start = datetime.datetime.strptime(start, '%H:%M:%S').time()
        stop = datetime.datetime.strptime(stop, '%H:%M:%S').time()
        now = datetime.datetime.now().time()

        dim = now > start and now < stop
        if not dim == self.dim:
            if dim:
                low = self.preferences.get("parameters", "brightness", "dim level")
                self.slave.startDetached(f"xbacklight -set {low}")
                self.lastLevel = self.level
            else:
                if self.level < self.lastLevel:
                    self.slave.startDetached(f"xbacklight -set {self.lastLevel}")
        self.dim = dim

    def _parse(self, output):
        self.level = int(output.split(".")[0])

    def update(self, output):
        self._parse(output)
        if not self.lastBrightness == self.level:
            if self.lastBrightness > -1:
                self.popup.emit(f"{self.level}%")

        if self.preferences.get("parameters", "brightness", "dim"):
            self._dim()

        self.lastBrightness = self.level
        self.ready = True
        self.fetch.emit()


class Main(QtWidgets.QSystemTrayIcon):
    popup = QtCore.pyqtSignal(object, str)
    error = QtCore.pyqtSignal(object, int)
    execute = QtCore.pyqtSignal(str)

    def __init__(self, parent):
        super().__init__()
        self.name = "brightness"
        self.preferences = parent.preferences
        self.slave = Slave(self)
        self.status = Status(self)
        self.fetchTimer = QtCore.QTimer(singleShot=True)

        self.slave.ready.connect(self._update)
        self.activated.connect(self._clickEvent)
        self.installEventFilter(self)
        self.status.popup.connect(self._popup)
        self.popup.connect(parent.display)

        self.setIcon(QtGui.QIcon(f"{LOCAL_DIR}../icons/{self.name}/default.svg"))
        if self.preferences.get("parameters", "brightness", "tray icon"):
            self.show()

    def _clickEvent(self, event):
        self._eventParser(event)

    def _cutoffDecrease(self, command):
        minimum = self.preferences.get("parameters", "brightness", "minimum") + 1
        length = len("xbacklight -dec ")
        try:
            self.slave.fetch("xbacklight -get")
            self.slave.waitForFinished()
            dec = int(command[length:])
            if (self.status.level - dec) <= minimum:
                command = f"xbacklight -set {minimum}"
        except ValueError:
            pass
        return command

    def _eventParser(self, event):
        trigger = trayEventParser(event)
        if trigger:
            enabled = self.preferences.get("actions", self.name, trigger)[0]
            if enabled:
                action = self.preferences.get("actions", self.name, trigger)[1].lower()
                command = self.preferences.get("exec", self.name, trigger)
                self.action(action, command)

    def _error(self, code):
        self.error.emit(self, code)

    def _fetch(self):
        self.slave.fetch("xbacklight -get")

    def _popup(self, message):
        self.popup.emit(self, message)

    def _update(self, output):
        self.status.update(output)
        self.setToolTip(f"Brightness: {self.status.level}%")

    def action(self, action, command=""):
        if action == "exec":
            if command.startswith("xbacklight -dec"):
                command = self._cutoffDecrease(command)
            self.execute.emit(command)

        elif action == "toggle brightness level":
            if self.status.level >= self.preferences.get("parameters", "brightness", "threshold"):
                self.slave.startDetached(self.preferences.get("parameters", "brightness", "low command"))
            else:
                self.slave.startDetached(self.preferences.get("parameters", "brightness", "high command"))

        else:
            self.log.error(f"Invalid action '{action}'")

    def eventFilter(self, object, event):
        self._eventParser(event)
        return QtWidgets.QSystemTrayIcon.event(self, event)
