#!/usr/bin/python3
import os
from PyQt5 import QtGui, QtWidgets, QtCore


class PopupWidget(QtWidgets.QWidget):
    done = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.preferences = parent.preferences
        self.icons = parent.icons
        self.iconLabel = QtWidgets.QLabel()
        self.textLabel = QtWidgets.QLabel()
        self.textLabel.setWordWrap(True)
        self.textLabel.setAlignment(QtCore.Qt.AlignCenter)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.iconLabel)
        layout.addWidget(self.textLabel)
        layout.setAlignment(self.iconLabel, QtCore.Qt.AlignCenter)
        layout.setAlignment(self.textLabel, QtCore.Qt.AlignCenter)
        self.setLayout(layout)

        self.locked = False
        self.displayQueue = []
        self.queueTimer = QtCore.QTimer(singleShot=True, interval=200)
        self.queueTimer.timeout.connect(self._displayQueued)
        self.hideTimer = QtCore.QTimer(singleShot=True)
        self.hideTimer.timeout.connect(self.hide)

        # Hide window from system taskbar
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint| QtCore.Qt.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WA_ShowWithoutActivating)
        if os.environ.get('DESKTOP_SESSION') == "openbox":
            self.setAttribute(QtCore.Qt.WA_X11NetWmWindowTypeToolBar)
        else:
            self.setAttribute(QtCore.Qt.WA_X11NetWmWindowTypeUtility)

        self.setMouseTracking(True)
        self.updateView()

    def _setGeometry(self):
        width = self.preferences.get("parameters", "popup", "width")
        height = self.preferences.get("parameters", "popup", "height")
        self.setFixedSize(width, height)

    def _setPosition(self):
        available = QtWidgets.QDesktopWidget().availableGeometry()
        position = self.frameGeometry()
        move = self.preferences.get("parameters", "popup", "position")

        if move == "top left":
            position.moveTopLeft(available.topLeft())
        elif move == "top center":
            position.moveCenter(available.center())
            position.moveTop(available.top())
        elif move == "top right":
            position.moveTopRight(available.topRight())
        elif move == "bottom left":
            position.moveBottomLeft(available.bottomLeft())
        elif move == "bottom center":
            position.moveCenter(available.center())
            position.moveBottom(available.bottom())
        elif move == "bottom right":
            position.moveBottomRight(available.bottomRight())
        elif move == "middle left":
            position.moveCenter(available.center())
            position.moveLeft(available.left())
        elif move == "middle center":
            position.moveCenter(available.center())
        elif move == "middle right":
            position.moveCenter(available.center())
            position.moveRight(available.right())

        x = position.x() + self.preferences.get("parameters", "popup", "horizontal offset")
        y = position.y() + self.preferences.get("parameters", "popup", "vertical offset")
        self.move(x, y)

    def _setStyle(self):
        fg = QtGui.QColor(self.preferences.get("parameters", "popup", "foreground"))
        bg = QtGui.QColor(self.preferences.get("parameters", "popup", "background"))
        palette = QtGui.QPalette()
        palette.setColor(QtGui.QPalette.WindowText, fg);
        palette.setColor(QtGui.QPalette.Window, bg);
        self.setPalette(palette)
        self.textLabel.setPalette(palette)
        self.setWindowOpacity(self.preferences.get("parameters", "popup", "opacity"))

        font = QtGui.QFont()
        font.setFamily(self.preferences.get("parameters", "popup", "font family"))
        font.setPointSize(self.preferences.get("parameters", "popup", "font size"))
        self.textLabel.setFont(font)

    def queue(self, sender, message):
        self.locked = True
        self.displayQueue.append((sender.name, sender.status.icon, message))
        self.queueTimer.start()

    def display(self, sender, message):
        if not self.locked:
            self._display((sender.name, sender.status.icon, message))

    def _display(self, content):
        if self.preferences.get("parameters", "popup", "enable"):
            width = int(self.preferences.get("parameters", "popup", "width") * 0.6)
            height = int(self.preferences.get("parameters", "popup", "height") * 0.6)
            icon = self.icons[f"{content[0]}/{content[1]}.svg"].pixmap(width, height)
            self.iconLabel.setPixmap(icon)
            self.textLabel.setText(content[2])

            duration = 1000 * self.preferences.get("parameters", "popup", "duration")
            self.hideTimer.start(duration)
            self.show()

    def _displayQueued(self):
        self._display(self.displayQueue[0])
        del self.displayQueue[0]

    def hideEvent(self, event):
        self.done.emit()
        if self.displayQueue:
            self.queueTimer.start()
        else:
            self.locked = False

    def mouseMoveEvent(self, event):
        self.hide()

    def mousePressEvent(self, event):
        self.hide()

    def updateView(self):
        self._setStyle()
        self._setGeometry()
        self._setPosition()
