#!/usr/bin/python3
import os
import sys
from PyQt5 import QtGui, QtWidgets, QtCore, QtMultimedia

try:
    from obsuite.backend.common import Slave
except ImportError:
    from backend.common import Slave

LOCAL_DIR = os.path.dirname(os.path.realpath(__file__)) + '/'


class Status(QtCore.QObject):
    popupQueued = QtCore.pyqtSignal(str)
    fetch = QtCore.pyqtSignal()
    play = QtCore.pyqtSignal()
    criticalTimerStart = QtCore.pyqtSignal(int)
    criticalTimerStop = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.preferences = parent.preferences
        self.ready = True
        self.lastLevel = 0
        self.warning = 0

    def _parse(self):
        thermometer = self.preferences.get("parameters", "temperature", "file")
        with open(thermometer) as f:
            self.level = int(f.read())/1000
            self.level = round(self.level)

    def update(self):
        criticalThreshold = self.preferences.get("parameters", "temperature", "critical threshold")
        highThreshold = self.preferences.get("parameters", "temperature", "high threshold")
        self._parse()

        if self.level >= criticalThreshold:
            self.icon = "critical"
        elif self.level >= highThreshold:
            self.icon = "high"
        else:
            self.icon = "low"

        if self.parent.criticalTimer.isActive() and self.level <= (criticalThreshold - 5):
            self.criticalTimerStop.emit()
            self.warning = 1
            self.popupQueued.emit(f"Critical timer stopped\n{self.level}°C")

        if self.warning > 0 and self.level <= (highThreshold - 5):
            self.warning = 0
            self.popupQueued.emit(f"{self.level}°C")

        elif self.warning < 2 and self.level >= criticalThreshold:
            delay = self.preferences.get("parameters", "temperature", "critical delay")
            self.play.emit()
            self.warning = 2
            self.criticalTimerStart.emit(delay * 60 * 1000)
            self.popupQueued.emit(f"Critical temperature\n{self.level}°C")

        elif self.warning < 1 and self.level >= highThreshold:
            self.warning = 1
            self.popupQueued.emit(f"High temperature\n{self.level}°C")

        self.lastLevel = self.level
        self.fetch.emit()


class Main(QtCore.QObject):
    popupQueued = QtCore.pyqtSignal(object, str)
    error = QtCore.pyqtSignal(object, int)

    def __init__(self, parent):
        super().__init__()
        self.name = "temperature"
        self.preferences = parent.preferences
        self.slave = Slave(self)
        self.status = Status(self)
        self.fetchTimer = QtCore.QTimer(singleShot=True)
        self.criticalTimer = QtCore.QTimer(singleShot=True)

        self.criticalTimer.timeout.connect(self._critical)
        self.status.criticalTimerStop.connect(self.criticalTimer.stop)
        self.status.criticalTimerStart.connect(self.criticalTimer.start)
        self.status.popupQueued.connect(self._popupQueued)
        self.popupQueued.connect(parent.displayQueued)

        self.enableCritical = self.preferences.get("parameters", "temperature", "critical")

    def _critical(self):
        if self.enableCritical:
            cmd = self.preferences.get("parameters", "temperature", "critical command")
            self.log.warning(f"Execution of '{cmd}'")
            self.slave.start(cmd)
            self.slave.waitForFinished()
            self.fetchTimer.start()  # Reset timer
        else:
            self.log.warning("Critical action ignored")
            delay = self.preferences.get("parameters", "temperature", "critical delay") * 60 * 1000
            self.criticalTimer.start(delay)  # Reset timer
            self.popupQueued.emit(self, f"Temperature ({self.status.level}%)")

    def _error(self, code):
        self.error.emit(self, code)

    def _fetch(self):
        if self._hasThermometer():
            self.status.update()

    def _hasThermometer(self):
        try:
            with open(self.preferences.get("parameters", "temperature", "file")) as f:
                int(f.read())
            return True
        except FileNotFoundError:
            self.error.emit(self, QtCore.QProcess.FailedToStart)
        except ValueError:
            self.error.emit(self, QtCore.QProcess.UnknownError)
        return False

    def _popupQueued(self, message):
        self.popupQueued.emit(self, message)
